# -*- coding: utf-8 -*-
#
# Copyright © 2013 Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
"""Spyder unitest plugin."""

# Local imports
from .unittestplugin import UnitTestPlugin as PLUGIN_CLASS

__version__ = '0.3.0'
PLUGIN_CLASS
